@@ -4,7 +4,6 @@ use libfuzzer_sys::{arbitrary, arbitrary::Arbitrary, fuzz_target};
 use wasm_smith::{Config, ConfiguredModule};
 use wasmer::{imports, Instance, Module, Store};
 use wasmer_compiler_singlepass::Singlepass;
-use wasmer_engine_universal::Universal;
 
 #[derive(Arbitrary, Debug, Default, Copy, Clone)]
 struct NoImportsConfig;
@@ -40,7 +39,7 @@ fuzz_target!(|module: WasmSmithModule| {
     }
 
     let compiler = Singlepass::default();
-    let store = Store::new(&Universal::new(compiler).engine());
+    let mut store = Store::new(compiler);
     let module = Module::new(&store, &wasm_bytes);
     let module = match module {
         Ok(m) => m,
@@ -52,7 +51,7 @@ fuzz_target!(|module: WasmSmithModule| {
             panic!("{}", e);
         }
     };
-    match Instance::new(&module, &imports! {}) {
+    match Instance::new(&mut store, &module, &imports! {}) {
         Ok(_) => {}
         Err(e) => {
             let error_message = format!("{}", e);
