@@ -533,6 +533,10 @@ pub enum wasmer_parser_operator_t {
     F32x4RelaxedMax,
     F64x2RelaxedMin,
     F64x2RelaxedMax,
+    I16x8RelaxedQ15mulrS,
+    I16x8DotI8x16I7x16S,
+    I32x4DotI8x16I7x16AddS,
+    F32x4RelaxedDotBf16x8AddF32x4,
 }
 
 impl<'a> From<&Operator<'a>> for wasmer_parser_operator_t {
@@ -1044,8 +1048,8 @@ impl<'a> From<&Operator<'a>> for wasmer_parser_operator_t {
             O::V128Store16Lane { .. } => Self::V128Store16Lane,
             O::V128Store32Lane { .. } => Self::V128Store32Lane,
             O::V128Store64Lane { .. } => Self::V128Store64Lane,
-            O::I8x16RoundingAverageU => Self::I8x16RoundingAverageU,
-            O::I16x8RoundingAverageU => Self::I16x8RoundingAverageU,
+            O::I8x16AvgrU => Self::I8x16RoundingAverageU,
+            O::I16x8AvgrU => Self::I16x8RoundingAverageU,
             O::I16x8Q15MulrSatS => Self::I16x8Q15MulrSatS,
             O::F32x4DemoteF64x2Zero => Self::F32x4DemoteF64x2Zero,
             O::F64x2PromoteLowF32x4 => Self::F64x2PromoteLowF32x4,
@@ -1058,18 +1062,22 @@ impl<'a> From<&Operator<'a>> for wasmer_parser_operator_t {
             O::I32x4RelaxedTruncSatF32x4U => Self::I32x4RelaxedTruncSatF32x4U,
             O::I32x4RelaxedTruncSatF64x2SZero => Self::I32x4RelaxedTruncSatF64x2SZero,
             O::I32x4RelaxedTruncSatF64x2UZero => Self::I32x4RelaxedTruncSatF64x2UZero,
-            O::F32x4Fma => Self::F32x4Fma,
-            O::F32x4Fms => Self::F32x4Fms,
-            O::F64x2Fma => Self::F64x2Fma,
-            O::F64x2Fms => Self::F64x2Fms,
-            O::I8x16LaneSelect => Self::I8x16LaneSelect,
-            O::I16x8LaneSelect => Self::I16x8LaneSelect,
-            O::I32x4LaneSelect => Self::I32x4LaneSelect,
-            O::I64x2LaneSelect => Self::I64x2LaneSelect,
+            O::F32x4RelaxedFma => Self::F32x4Fma,
+            O::F32x4RelaxedFnma => Self::F32x4Fms,
+            O::F64x2RelaxedFma => Self::F64x2Fma,
+            O::F64x2RelaxedFnma => Self::F64x2Fms,
+            O::I8x16RelaxedLaneselect => Self::I8x16LaneSelect,
+            O::I16x8RelaxedLaneselect => Self::I16x8LaneSelect,
+            O::I32x4RelaxedLaneselect => Self::I32x4LaneSelect,
+            O::I64x2RelaxedLaneselect => Self::I64x2LaneSelect,
             O::F32x4RelaxedMin => Self::F32x4RelaxedMin,
             O::F32x4RelaxedMax => Self::F32x4RelaxedMax,
             O::F64x2RelaxedMin => Self::F64x2RelaxedMin,
             O::F64x2RelaxedMax => Self::F64x2RelaxedMax,
+            O::I16x8RelaxedQ15mulrS => Self::I16x8RelaxedQ15mulrS,
+            O::I16x8DotI8x16I7x16S => Self::I16x8DotI8x16I7x16S,
+            O::I32x4DotI8x16I7x16AddS => Self::I32x4DotI8x16I7x16AddS,
+            O::F32x4RelaxedDotBf16x8AddF32x4 => Self::F32x4RelaxedDotBf16x8AddF32x4,
         }
     }
 }
