@@ -1,6 +1,7 @@
-use std::fs;
 use std::io::{self, Write};
 use std::path::{Path, PathBuf};
+use std::time::Duration;
+use std::{fs, io::IsTerminal};
 
 use anyhow::{anyhow, bail, Context};
 use flate2::{write::GzEncoder, Compression};
@@ -9,7 +10,7 @@ use tar::Builder;
 use thiserror::Error;
 use time::{self, OffsetDateTime};
 
-use crate::publish::SignArchiveResult;
+use crate::{package::builder::validate::ValidationPolicy, publish::SignArchiveResult};
 use crate::{WasmerConfig, PACKAGE_TOML_FALLBACK_NAME};
 
 const MIGRATIONS: &[(i32, &str)] = &[
@@ -32,12 +33,16 @@ pub struct Publish {
     pub package_name: Option<String>,
     /// Override the package version of the uploaded package in the wasmer.toml
     pub version: Option<semver::Version>,
-    /// Override the token (by default, it will use the current logged in user)
-    pub token: Option<String>,
+    /// The auth token to use.
+    pub token: String,
     /// Skip validation of the uploaded package
     pub no_validate: bool,
     /// Directory containing the `wasmer.toml` (defaults to current root dir)
     pub package_path: Option<String>,
+    /// Wait for package to be available on the registry before exiting
+    pub wait: bool,
+    /// Timeout (in seconds) for the publish query to the registry
+    pub timeout: Duration,
 }
 
 #[derive(Debug, Error)]
@@ -129,8 +134,16 @@ impl Publish {
             }
         };
 
-        if !self.no_validate {
-            validate::validate_directory(&manifest, &registry, manifest_dir)?;
+        let mut policy = self.validation_policy();
+
+        if !policy.skip_validation() {
+            validate::validate_directory(
+                &manifest,
+                &registry,
+                manifest_dir,
+                &mut *policy,
+                &self.token,
+            )?;
         }
 
         let archive_path = &archive_meta.archive_path;
@@ -162,7 +175,7 @@ impl Publish {
 
         crate::publish::try_chunked_uploading(
             Some(registry),
-            self.token.clone(),
+            Some(self.token.clone()),
             &manifest.package,
             &archive_meta.manifest_toml,
             &archive_meta.license,
@@ -178,8 +191,20 @@ impl Publish {
             &maybe_signature_data,
             archived_data_size,
             self.quiet,
+            self.wait,
+            self.timeout,
         )
     }
+
+    fn validation_policy(&self) -> Box<dyn ValidationPolicy> {
+        if self.no_validate {
+            Box::<validate::Skip>::default()
+        } else if std::io::stdin().is_terminal() {
+            Box::<validate::Interactive>::default()
+        } else {
+            Box::<validate::NonInteractive>::default()
+        }
+    }
 }
 
 struct ConstructedPackageArchive {
@@ -577,81 +602,38 @@ mod validate {
     use wasmer_wasm_interface::{validate, Interface};
 
     use super::interfaces;
-    use crate::interface::InterfaceFromServer;
+    use crate::{interface::InterfaceFromServer, QueryPackageError};
     use std::{
         fs,
         io::Read,
+        ops::ControlFlow,
         path::{Path, PathBuf},
     };
 
-    pub fn validate_directory(
+    pub(crate) fn validate_directory(
         manifest: &wasmer_toml::Manifest,
         registry: &str,
         pkg_path: PathBuf,
+        callbacks: &mut dyn ValidationPolicy,
+        auth_token: &str,
     ) -> anyhow::Result<()> {
         // validate as dir
         for module in manifest.modules.iter() {
-            let source_path = if module.source.is_relative() {
-                pkg_path.join(&module.source)
+            if let Err(e) = validate_module(module, registry, &pkg_path) {
+                if callbacks.on_invalid_module(module, &e).is_break() {
+                    return Err(e.into());
+                }
+            }
+        }
+
+        if would_change_package_privacy(manifest, registry, auth_token)?
+            && callbacks.on_package_privacy_changed(manifest).is_break()
+        {
+            if manifest.package.private {
+                return Err(ValidationError::WouldBecomePrivate.into());
             } else {
-                module.source.clone()
-            };
-            let source_path_string = source_path.to_string_lossy().to_string();
-            let mut wasm_file =
-                fs::File::open(&source_path).map_err(|_| ValidationError::MissingFile {
-                    file: source_path_string.clone(),
-                })?;
-            let mut wasm_buffer = Vec::new();
-            wasm_file.read_to_end(&mut wasm_buffer).map_err(|err| {
-                ValidationError::MiscCannotRead {
-                    file: source_path_string.clone(),
-                    error: format!("{}", err),
-                }
-            })?;
-
-            if let Some(bindings) = &module.bindings {
-                validate_bindings(bindings, &pkg_path)?;
+                return Err(ValidationError::WouldBecomePublic.into());
             }
-
-            // hack, short circuit if no interface for now
-            if module.interfaces.is_none() {
-                return validate_wasm_and_report_errors_old(&wasm_buffer[..], source_path_string);
-            }
-
-            let mut conn = super::open_db()?;
-            let mut interface: Interface = Default::default();
-            for (interface_name, interface_version) in
-                module.interfaces.clone().unwrap_or_default().into_iter()
-            {
-                if !interfaces::interface_exists(&mut conn, &interface_name, &interface_version)? {
-                    // download interface and store it if we don't have it locally
-                    let interface_data_from_server = InterfaceFromServer::get(
-                        registry,
-                        interface_name.clone(),
-                        interface_version.clone(),
-                    )?;
-                    interfaces::import_interface(
-                        &mut conn,
-                        &interface_name,
-                        &interface_version,
-                        &interface_data_from_server.content,
-                    )?;
-                }
-                let sub_interface = interfaces::load_interface_from_db(
-                    &mut conn,
-                    &interface_name,
-                    &interface_version,
-                )?;
-                interface = interface
-                    .merge(sub_interface)
-                    .map_err(|e| anyhow!("Failed to merge interface {}: {}", &interface_name, e))?;
-            }
-            validate::validate_wasm_and_report_errors(&wasm_buffer, &interface).map_err(|e| {
-                ValidationError::InvalidWasm {
-                    file: source_path_string,
-                    error: format!("{:?}", e),
-                }
-            })?;
         }
 
         log::debug!("package at path {:#?} validated", &pkg_path);
@@ -659,6 +641,114 @@ mod validate {
         Ok(())
     }
 
+    /// Check if publishing this manifest would change the package's privacy.
+    fn would_change_package_privacy(
+        manifest: &wasmer_toml::Manifest,
+        registry: &str,
+        auth_token: &str,
+    ) -> Result<bool, ValidationError> {
+        let result = crate::query_package_from_registry(
+            registry,
+            &manifest.package.name,
+            None,
+            Some(auth_token),
+        );
+
+        match result {
+            Ok(package_version) => Ok(package_version.is_private != manifest.package.private),
+            Err(QueryPackageError::NoPackageFound { .. }) => {
+                // The package hasn't been published yet
+                Ok(false)
+            }
+            Err(e) => Err(e.into()),
+        }
+    }
+
+    fn validate_module(
+        module: &wasmer_toml::Module,
+        registry: &str,
+        pkg_path: &Path,
+    ) -> Result<(), ValidationError> {
+        let source_path = if module.source.is_relative() {
+            pkg_path.join(&module.source)
+        } else {
+            module.source.clone()
+        };
+        let source_path_string = source_path.to_string_lossy().to_string();
+        let mut wasm_file =
+            fs::File::open(&source_path).map_err(|_| ValidationError::MissingFile {
+                file: source_path_string.clone(),
+            })?;
+        let mut wasm_buffer = Vec::new();
+        wasm_file
+            .read_to_end(&mut wasm_buffer)
+            .map_err(|err| ValidationError::MiscCannotRead {
+                file: source_path_string.clone(),
+                error: format!("{}", err),
+            })?;
+
+        if let Some(bindings) = &module.bindings {
+            validate_bindings(bindings, pkg_path)?;
+        }
+
+        // hack, short circuit if no interface for now
+        if module.interfaces.is_none() {
+            return validate_wasm_and_report_errors_old(&wasm_buffer[..], source_path_string);
+        }
+
+        let mut conn = super::open_db().map_err(ValidationError::UpdatingInterfaces)?;
+        let mut interface: Interface = Default::default();
+        for (interface_name, interface_version) in
+            module.interfaces.clone().unwrap_or_default().into_iter()
+        {
+            add_module_interface(
+                &mut conn,
+                interface_name,
+                interface_version,
+                registry,
+                &mut interface,
+            )
+            .map_err(ValidationError::UpdatingInterfaces)?;
+        }
+        validate::validate_wasm_and_report_errors(&wasm_buffer, &interface).map_err(|e| {
+            ValidationError::InvalidWasm {
+                file: source_path_string,
+                error: format!("{:?}", e),
+            }
+        })?;
+
+        Ok(())
+    }
+
+    fn add_module_interface(
+        conn: &mut rusqlite::Connection,
+        interface_name: String,
+        interface_version: String,
+        registry: &str,
+        interface: &mut Interface,
+    ) -> anyhow::Result<()> {
+        if !interfaces::interface_exists(conn, &interface_name, &interface_version)? {
+            // download interface and store it if we don't have it locally
+            let interface_data_from_server = InterfaceFromServer::get(
+                registry,
+                interface_name.clone(),
+                interface_version.clone(),
+            )?;
+            interfaces::import_interface(
+                conn,
+                &interface_name,
+                &interface_version,
+                &interface_data_from_server.content,
+            )?;
+        }
+        let sub_interface =
+            interfaces::load_interface_from_db(conn, &interface_name, &interface_version)?;
+        *interface = interface
+            .merge(sub_interface)
+            .map_err(|e| anyhow!("Failed to merge interface {}: {}", &interface_name, e))?;
+        Ok(())
+    }
+
     fn validate_bindings(
         bindings: &wasmer_toml::Bindings,
         base_directory_path: &Path,
@@ -670,6 +760,7 @@ mod validate {
     }
 
     #[derive(Debug, Error)]
+    #[non_exhaustive]
     pub enum ValidationError {
         #[error("WASM file \"{file}\" detected as invalid because {error}")]
         InvalidWasm { file: String, error: String },
@@ -679,13 +770,21 @@ mod validate {
         MiscCannotRead { file: String, error: String },
         #[error(transparent)]
         Imports(#[from] wasmer_toml::ImportsError),
+        #[error("Unable to update the interfaces database")]
+        UpdatingInterfaces(#[source] anyhow::Error),
+        #[error("Aborting because publishing the package would make it public")]
+        WouldBecomePublic,
+        #[error("Aborting because publishing the package would make it private")]
+        WouldBecomePrivate,
+        #[error("Unable to look up package information")]
+        Registry(#[from] QueryPackageError),
     }
 
     // legacy function, validates wasm.  TODO: clean up
     pub fn validate_wasm_and_report_errors_old(
         wasm: &[u8],
         file_name: String,
-    ) -> anyhow::Result<()> {
+    ) -> Result<(), ValidationError> {
         use wasmparser::WasmDecoder;
         let mut parser = wasmparser::ValidatingParser::new(
             wasm,
@@ -707,13 +806,144 @@ mod validate {
                     return Err(ValidationError::InvalidWasm {
                         file: file_name,
                         error: format!("{}", e),
-                    }
-                    .into());
+                    });
                 }
                 _ => {}
             }
         }
     }
+
+    /// How should validation be treated by the publishing process?
+    pub(crate) trait ValidationPolicy {
+        /// Should validation be skipped entirely?
+        fn skip_validation(&mut self) -> bool;
+
+        /// How should publishing proceed when a module is invalid?
+        fn on_invalid_module(
+            &mut self,
+            module: &wasmer_toml::Module,
+            error: &ValidationError,
+        ) -> ControlFlow<(), ()>;
+
+        /// How should publishing proceed when it might change a package's
+        /// privacy? (i.e. by making a private package publicly available).
+        fn on_package_privacy_changed(
+            &mut self,
+            manifest: &wasmer_toml::Manifest,
+        ) -> ControlFlow<(), ()>;
+    }
+
+    #[derive(Debug, Default, Copy, Clone, PartialEq)]
+    pub(crate) struct Skip;
+
+    impl ValidationPolicy for Skip {
+        fn skip_validation(&mut self) -> bool {
+            true
+        }
+
+        fn on_invalid_module(
+            &mut self,
+            _module: &wasmer_toml::Module,
+            _error: &ValidationError,
+        ) -> ControlFlow<(), ()> {
+            unreachable!()
+        }
+
+        fn on_package_privacy_changed(
+            &mut self,
+            _manifest: &wasmer_toml::Manifest,
+        ) -> ControlFlow<(), ()> {
+            unreachable!()
+        }
+    }
+
+    #[derive(Debug, Default, Copy, Clone, PartialEq)]
+    pub(crate) struct Interactive;
+
+    impl ValidationPolicy for Interactive {
+        fn skip_validation(&mut self) -> bool {
+            false
+        }
+
+        fn on_invalid_module(
+            &mut self,
+            module: &wasmer_toml::Module,
+            error: &ValidationError,
+        ) -> ControlFlow<(), ()> {
+            let module_name = &module.name;
+            let prompt =
+                format!("Validation error with the \"{module_name}\" module: {error}. Would you like to continue?");
+
+            match dialoguer::Confirm::new()
+                .with_prompt(prompt)
+                .default(false)
+                .interact()
+            {
+                Ok(true) => ControlFlow::Continue(()),
+                Ok(false) => ControlFlow::Break(()),
+                Err(e) => {
+                    tracing::error!(
+                        error = &e as &dyn std::error::Error,
+                        "Unable to check whether the user wants to change the package's privacy",
+                    );
+                    ControlFlow::Break(())
+                }
+            }
+        }
+
+        fn on_package_privacy_changed(
+            &mut self,
+            manifest: &wasmer_toml::Manifest,
+        ) -> ControlFlow<(), ()> {
+            let privacy = if manifest.package.private {
+                "private"
+            } else {
+                "public"
+            };
+            let prompt =
+                format!("This will make the package {privacy}. Would you like to continue?");
+
+            match dialoguer::Confirm::new()
+                .with_prompt(prompt)
+                .default(false)
+                .interact()
+            {
+                Ok(true) => ControlFlow::Continue(()),
+                Ok(false) => ControlFlow::Break(()),
+                Err(e) => {
+                    tracing::error!(
+                        error = &e as &dyn std::error::Error,
+                        "Unable to check whether the user wants to change the package's privacy",
+                    );
+                    ControlFlow::Break(())
+                }
+            }
+        }
+    }
+
+    #[derive(Debug, Default, Copy, Clone, PartialEq)]
+    pub(crate) struct NonInteractive;
+
+    impl ValidationPolicy for NonInteractive {
+        fn skip_validation(&mut self) -> bool {
+            false
+        }
+
+        fn on_invalid_module(
+            &mut self,
+            _module: &wasmer_toml::Module,
+            _error: &ValidationError,
+        ) -> ControlFlow<(), ()> {
+            ControlFlow::Break(())
+        }
+
+        fn on_package_privacy_changed(
+            &mut self,
+            _manifest: &wasmer_toml::Manifest,
+        ) -> ControlFlow<(), ()> {
+            ControlFlow::Break(())
+        }
+    }
 }
 
 #[cfg(test)]
@@ -750,11 +980,11 @@ runner = "https://webc.org/runner/wcgi"
         std::fs::write(&manifest_path, manifest_str).unwrap();
         std::fs::write(mp.join("module.wasm"), "()").unwrap();
 
-        let manifest = wasmer_toml::Manifest::parse(&manifest_str).unwrap();
+        let manifest = wasmer_toml::Manifest::parse(manifest_str).unwrap();
 
-        let meta = construct_tar_gz(&archive_dir.path(), &manifest, &manifest_path).unwrap();
+        let meta = construct_tar_gz(archive_dir.path(), &manifest, &manifest_path).unwrap();
 
-        let mut data = std::io::Cursor::new(std::fs::read(&meta.archive_path).unwrap());
+        let mut data = std::io::Cursor::new(std::fs::read(meta.archive_path).unwrap());
 
         let gz = flate2::read::GzDecoder::new(&mut data);
         let mut archive = tar::Archive::new(gz);
@@ -809,10 +1039,10 @@ exports = "crum-sort.wai"
         std::fs::write(mp.join("crumsort_wasm.wasm"), "()").unwrap();
         std::fs::write(mp.join("crum-sort.wai"), "/// crum-sort.wai").unwrap();
 
-        let manifest = wasmer_toml::Manifest::parse(&manifest_str).unwrap();
-        let meta = construct_tar_gz(&archive_dir.path(), &manifest, &manifest_path).unwrap();
+        let manifest = wasmer_toml::Manifest::parse(manifest_str).unwrap();
+        let meta = construct_tar_gz(archive_dir.path(), &manifest, &manifest_path).unwrap();
 
-        let mut data = std::io::Cursor::new(std::fs::read(&meta.archive_path).unwrap());
+        let mut data = std::io::Cursor::new(std::fs::read(meta.archive_path).unwrap());
 
         let gz = flate2::read::GzDecoder::new(&mut data);
         let mut archive = tar::Archive::new(gz);
