@@ -3,9 +3,11 @@ mod guard;
 mod interest;
 #[cfg(feature = "sys")]
 mod selector;
+pub mod waker;
 
 #[cfg(feature = "sys")]
 pub use guard::*;
 pub use interest::*;
 #[cfg(feature = "sys")]
 pub use selector::*;
+pub use waker::*;
