@@ -20,7 +20,10 @@ pub mod wasi;
 pub mod wasix;
 
 use bytes::{Buf, BufMut};
-use futures::Future;
+use futures::{
+    future::{BoxFuture, LocalBoxFuture},
+    Future,
+};
 use tracing::instrument;
 pub use wasi::*;
 pub use wasix::*;
@@ -47,7 +50,7 @@ pub(crate) use std::{
     thread::LocalKey,
     time::Duration,
 };
-use std::{io::IoSlice, marker::PhantomData, mem::MaybeUninit, time::Instant};
+use std::{io::IoSlice, marker::PhantomData, mem::MaybeUninit, task::Waker, time::Instant};
 
 pub(crate) use bytes::{Bytes, BytesMut};
 pub(crate) use cooked_waker::IntoWaker;
@@ -100,7 +103,7 @@ pub(crate) use crate::{
         socket::{InodeHttpSocketType, InodeSocket, InodeSocketKind},
         write_ip_port,
     },
-    runtime::{task_manager::VirtualTaskManagerExt, SpawnMemoryType},
+    runtime::SpawnMemoryType,
     state::{
         self, iterate_poll_events, InodeGuard, InodeWeakGuard, PollEvent, PollEventBuilder,
         WasiFutex, WasiState,
@@ -115,6 +118,7 @@ use crate::{
         MAX_SYMLINKS,
     },
     os::task::thread::RewindResult,
+    runtime::task_manager::InlineWaker,
     utils::store::InstanceSnapshot,
     DeepSleepWork, RewindPostProcess, RewindState, SpawnError, WasiInodes,
 };
@@ -218,16 +222,19 @@ pub(crate) fn read_bytes<T: Read, M: MemorySize>(
 
 // TODO: remove allow once inodes are refactored (see comments on [`WasiState`])
 #[allow(clippy::await_holding_lock)]
-pub async unsafe fn stderr_write(
+pub unsafe fn stderr_write<'a>(
     ctx: &FunctionEnvMut<'_, WasiEnv>,
     buf: &[u8],
-) -> Result<(), Errno> {
+) -> LocalBoxFuture<'a, Result<(), Errno>> {
     let env = ctx.data();
     let (memory, state, inodes) = env.get_memory_and_wasi_state_and_inodes(ctx, 0);
 
-    let mut stderr = WasiInodes::stderr_mut(&state.fs.fd_map).map_err(fs_error_into_wasi_err)?;
-
-    stderr.write_all(buf).await.map_err(map_io_err)
+    let buf = buf.to_vec();
+    let fd_map = state.fs.fd_map.clone();
+    Box::pin(async move {
+        let mut stderr = WasiInodes::stderr_mut(&fd_map).map_err(fs_error_into_wasi_err)?;
+        stderr.write_all(&buf).await.map_err(map_io_err)
+    })
 }
 
 fn block_on_with_timeout<T, Fut>(
@@ -267,7 +274,6 @@ where
     if nonblocking {
         let waker = WasiDummyWaker.into_waker();
         let mut cx = Context::from_waker(&waker);
-        let _guard = tasks.runtime_enter();
         let mut pinned_work = Box::pin(work);
         if let Poll::Ready(res) = pinned_work.as_mut().poll(&mut cx) {
             return res;
@@ -276,7 +282,7 @@ where
     }
 
     // Slow path, block on the work and process process
-    tasks.block_on(work)
+    InlineWaker::block_on(work)
 }
 
 /// Asyncify takes the current thread and blocks on the async runtime associated with it
@@ -316,9 +322,6 @@ where
             if let Poll::Ready(res) = Pin::new(&mut self.pinned_work).poll(cx) {
                 return Poll::Ready(Ok(res));
             }
-            if let Some(exit_code) = self.ctx.data().should_exit() {
-                return Poll::Ready(Err(WasiError::Exit(exit_code)));
-            }
             if let Some(signals) = self.ctx.data().thread.pop_signals_or_subscribe(cx.waker()) {
                 if let Err(err) = WasiEnv::process_signals_internal(self.ctx, signals) {
                     return Poll::Ready(Err(err));
@@ -342,15 +345,15 @@ where
 pub type AsyncifyFuture = dyn Future<Output = Bytes> + Send + Sync + 'static;
 
 // This poller will process any signals when the main working function is idle
-struct AsyncifyPoller<'a, T, Fut>
+struct AsyncifyPoller<'a, 'b, 'c, T, Fut>
 where
     Fut: Future<Output = T> + Send + Sync + 'static,
 {
     process_signals: bool,
-    thread: WasiThread,
+    ctx: &'b mut FunctionEnvMut<'c, WasiEnv>,
     work: &'a mut Pin<Box<Fut>>,
 }
-impl<'a, T, Fut> Future for AsyncifyPoller<'a, T, Fut>
+impl<'a, 'b, 'c, T, Fut> Future for AsyncifyPoller<'a, 'b, 'c, T, Fut>
 where
     Fut: Future<Output = T> + Send + Sync + 'static,
 {
@@ -360,21 +363,23 @@ where
         if let Poll::Ready(res) = self.work.as_mut().poll(cx) {
             return Poll::Ready(Ok(res));
         }
-        if let Some(forced_exit) = self.thread.try_join() {
+
+        let env = self.ctx.data();
+        if let Some(forced_exit) = env.thread.try_join() {
             return Poll::Ready(Err(WasiError::Exit(forced_exit.unwrap_or_else(|err| {
                 tracing::debug!("exit runtime error - {}", err);
                 Errno::Child.into()
             }))));
         }
-        if self.process_signals && self.thread.has_signals_or_subscribe(cx.waker()) {
-            let signals = self.thread.signals().lock().unwrap();
+        if self.process_signals && env.thread.has_signals_or_subscribe(cx.waker()) {
+            let signals = env.thread.signals().lock().unwrap();
             for sig in signals.0.iter() {
                 if *sig == Signal::Sigint
                     || *sig == Signal::Sigquit
                     || *sig == Signal::Sigkill
                     || *sig == Signal::Sigabrt
                 {
-                    let exit_code = self.thread.set_or_get_exit_code_for_signal(*sig);
+                    let exit_code = env.thread.set_or_get_exit_code_for_signal(*sig);
                     return Poll::Ready(Err(WasiError::Exit(exit_code)));
                 }
             }
@@ -402,9 +407,9 @@ pub enum AsyncifyAction<'a, R> {
 /// and instead process it on a shared task
 ///
 pub(crate) fn __asyncify_with_deep_sleep<M: MemorySize, T, Fut>(
-    ctx: FunctionEnvMut<'_, WasiEnv>,
+    mut ctx: FunctionEnvMut<'_, WasiEnv>,
     deep_sleep_time: Duration,
-    trigger: Fut,
+    work: Fut,
 ) -> Result<AsyncifyAction<'_, T>, WasiError>
 where
     T: serde::Serialize + serde::de::DeserializeOwned,
@@ -418,7 +423,7 @@ where
         .unwrap_or(true);
 
     // Box up the trigger
-    let mut trigger = Box::pin(trigger);
+    let mut trigger = Box::pin(work);
 
     // Define the work
     let tasks = ctx.data().tasks().clone();
@@ -426,9 +431,14 @@ where
         let env = ctx.data();
 
         // Create the deep sleeper
+        let tasks_for_deep_sleep = if env.enable_deep_sleep {
+            Some(env.tasks().clone())
+        } else {
+            None
+        };
         let deep_sleep_wait = async {
-            if env.enable_deep_sleep {
-                env.tasks().sleep_now(deep_sleep_time).await
+            if let Some(tasks) = tasks_for_deep_sleep {
+                tasks.sleep_now(deep_sleep_time).await
             } else {
                 InfiniteSleep::default().await
             }
@@ -438,7 +448,7 @@ where
             // Inner wait with finializer
             res = AsyncifyPoller {
                 process_signals,
-                thread: ctx.data().thread.clone(),
+                ctx: &mut ctx,
                 work: &mut trigger,
             } => {
                 let result = res?;
@@ -458,8 +468,9 @@ where
         })
     };
 
-    // Block on the work
-    tasks.block_on(work)
+    // Block until the work is finished or until we
+    // unload the thread using asyncify
+    InlineWaker::block_on(work)
 }
 
 /// Asyncify takes the current thread and blocks on the async runtime associated with it
@@ -495,17 +506,16 @@ where
             if let Some(exit_code) = self.env.should_exit() {
                 return Poll::Ready(Err(WasiError::Exit(exit_code)));
             }
-            if let Some(signals) = self.env.thread.pop_signals_or_subscribe(cx.waker()) {
+            if self.env.thread.has_signals_or_subscribe(cx.waker()) {
                 return Poll::Ready(Ok(Err(Errno::Intr)));
             }
             Poll::Pending
         }
     }
 
-    // Block on the work
-    let mut pinned_work = Box::pin(work);
-    let poller = Poller { env, pinned_work };
-    block_on_with_timeout(env.tasks(), timeout, poller)
+    // Block until the work is finished or until we
+    // unload the thread using asyncify
+    Ok(InlineWaker::block_on(work))
 }
 
 // This should be compiled away, it will simply wait forever however its never
@@ -537,11 +547,11 @@ where
         return Err(Errno::Access);
     }
 
-    let work = {
+    let mut work = {
         let inode = fd_entry.inode.clone();
         let tasks = env.tasks().clone();
-        let mut guard = inode.read();
-        match guard.deref() {
+        let mut guard = inode.write();
+        match guard.deref_mut() {
             Kind::Socket { socket } => {
                 // Clone the socket and release the lock
                 let socket = socket.clone();
@@ -556,8 +566,9 @@ where
         }
     };
 
-    // Block on the work and process it
-    env.tasks().block_on(work)
+    // Block until the work is finished or until we
+    // unload the thread using asyncify
+    InlineWaker::block_on(work)
 }
 
 /// Performs mutable work on a socket under an asynchronous runtime with
@@ -589,10 +600,11 @@ where
             drop(guard);
 
             // Start the work using the socket
-            let work = actor(socket, fd_entry);
+            let mut work = actor(socket, fd_entry);
 
-            // Block on the work and process it
-            tasks.block_on(work)
+            // Otherwise we block on the work and process it
+            // using an asynchronou context
+            InlineWaker::block_on(work)
         }
         _ => Err(Errno::Notsock),
     }
@@ -621,8 +633,8 @@ where
     let inode = fd_entry.inode.clone();
 
     let tasks = env.tasks().clone();
-    let mut guard = inode.read();
-    match guard.deref() {
+    let mut guard = inode.write();
+    match guard.deref_mut() {
         Kind::Socket { socket } => {
             // Clone the socket and release the lock
             let socket = socket.clone();
@@ -709,17 +721,13 @@ where
                 let work = actor(socket);
 
                 // Block on the work and process it
-                let (tx, rx) = std::sync::mpsc::channel();
-                tasks.block_on(Box::pin(async move {
-                    let ret = work.await;
-                    tx.send(ret);
-                }));
-                let new_socket = rx.recv().unwrap()?;
+                let res = InlineWaker::block_on(work);
+                let new_socket = res?;
 
                 if let Some(mut new_socket) = new_socket {
                     let mut guard = inode.write();
                     match guard.deref_mut() {
-                        Kind::Socket { socket } => {
+                        Kind::Socket { socket, .. } => {
                             std::mem::swap(socket, &mut new_socket);
                         }
                         _ => {
@@ -1267,7 +1275,7 @@ pub(crate) fn _prepare_wasi(wasi_env: &mut WasiEnv, args: Option<Vec<String>>) {
     }
 }
 
-pub(crate) fn conv_spawn_err_to_errno(err: SpawnError) -> Errno {
+pub(crate) fn conv_spawn_err_to_errno(err: &SpawnError) -> Errno {
     match err {
         SpawnError::AccessDenied => Errno::Access,
         SpawnError::NotFound => Errno::Noent,
@@ -1276,6 +1284,6 @@ pub(crate) fn conv_spawn_err_to_errno(err: SpawnError) -> Errno {
     }
 }
 
-pub(crate) fn conv_spawn_err_to_exit_code(err: SpawnError) -> ExitCode {
+pub(crate) fn conv_spawn_err_to_exit_code(err: &SpawnError) -> ExitCode {
     conv_spawn_err_to_errno(err).into()
 }
