@@ -11,7 +11,7 @@ use crate::syscalls::*;
 /// Output:
 /// - `Timestamp *time`
 ///     The value of the clock in nanoseconds
-#[instrument(level = "trace", skip_all, fields(?clock_id, %precision), ret)]
+//#[instrument(level = "trace", skip_all, fields(?clock_id, %precision), ret)]
 pub fn clock_time_get<M: MemorySize>(
     ctx: FunctionEnvMut<'_, WasiEnv>,
     clock_id: Snapshot0Clockid,
